// Car Movement

#include "CarMovementInit.as"
#include "Particles.as"

void onTick(CMovement@ this)
{
	CBlob@ blob = this.getBlob();
	CarMoveVars@ moveVars;
	if (!blob.get("moveVars", @moveVars))
	{
		return;
	}

	const bool left		= blob.isKeyPressed(key_left);
	const bool right	= blob.isKeyPressed(key_right);
	const bool up		= blob.isKeyPressed(key_up);
	const bool down		= blob.isKeyPressed(key_down);

	const bool is_client = getNet().isClient();

	CMap@ map = blob.getMap();
	Vec2f vel = blob.getVelocity();
	Vec2f pos = blob.getPosition();
	CShape@ shape = blob.getShape();	
	Vec2f aim = blob.getAimPos();

	const f32 vellen = shape.vellen;
	const bool onground = blob.isOnGround();

	f32 angle = blob.getAngleDegrees();
	Vec2f force = Vec2f(0, 0);
	//f32 torque = 0.25f;

	Vec2f velocity_vec = vel.RotateBy(angle * -1);	 	
 	float velocity_angle = (velocity_vec.getAngleDegrees()-90.0f);

 	if (velocity_angle < 0.0f)
		velocity_angle = Maths::Abs(velocity_angle);

	f32 dif = Maths::Abs(velocity_angle-180.0f); //slide amount
	if (dif > 90)
		dif -= 180;
	if (dif < 0)
		dif = dif*-1;

	Vec2f exaustoffset = Vec2f(velocity_vec.x*2, 14.0f-(vellen*1.5));

	if (is_client)
	{
		blob.getShape().setDrag(0.99f + (dif*0.006f));

		if (blob.isKeyJustPressed(key_action3))
	  	{ blob.set_u32("brake time", getGameTime()); } // Set to 0 ticks ~ only if key just pressed 

	  	u32 ticksSinceBrake = getGameTime() - blob.get_u32("brake time");

		f32 extra_drag = 0.99f + (ticksSinceBrake*0.08);

		if(blob.isKeyPressed(key_action3))
		{
			blob.getShape().setDrag(extra_drag);
		}
		else if (blob.isKeyJustReleased(key_action3))
		{
			blob.getShape().setDrag(0.99);
		}
		
		f32 currentVelocity = vel.Length();

		switch (moveVars.currentGear)
	 	{
	 		case 0: moveVars.GearMinRPM =  -20; moveVars.GearMaxRPM =  200; break;
	 		case 1: moveVars.GearMinRPM =  200; moveVars.GearMaxRPM =  400; break;
	 		case 2: moveVars.GearMinRPM =  400; moveVars.GearMaxRPM =  600; break;
	 		case 3: moveVars.GearMinRPM =  600; moveVars.GearMaxRPM =  800; break;
	 		case 4: moveVars.GearMinRPM =  800; moveVars.GearMaxRPM = 1000; break;
	 		case 5: moveVars.GearMinRPM = 1000; moveVars.GearMaxRPM = 1200; break;
	 	}

		if (up)
		{		
			moveVars.rpmAmount++;
			//moveVars.rpmAmount+= 6.0f;

		 if (moveVars.accelAmount+currentVelocity < moveVars.maxVel)
		 	{moveVars.accelAmount++;}

		 if (moveVars.rpmAmount*1 == moveVars.GearMaxRPM)
		 	{moveVars.currentGear+=1; }

		 	
		 	print("g "+moveVars.currentGear);

		 //print("mv:acamount"+moveVars.accelAmount);	

		 force.y -= moveVars.accelAmount*0.7f; // Thrust amount//
		 force.RotateBy(blob.getShape().getAngleDegrees());

		 exaustoffset.RotateBy(angle);
		 makeSmokeParticle(pos + exaustoffset + getRandomVelocity(2.0f, 2.0f, 180.0f));	

		 //makeSkidParticle(pos);			
		 //blob.getSprite().PlaySound("/enginerunning.ogg");
		 
	 	blob.getSprite().SetEmitSound("enginegoing.ogg");
		blob.getSprite().SetEmitSoundPaused(false);
		blob.getSprite().SetEmitSoundSpeed(moveVars.rpmAmount*0.01f);
		 
		 force *= moveVars.thrustFactor * moveVars.overallScale * 1.0f;
		 blob.AddForce(force);
		}
		else if (moveVars.rpmAmount > 0)
		{
			blob.getSprite().SetEmitSoundPaused(true);

			moveVars.rpmAmount--;
			//moveVars.rpmAmount-=20.0f;

			if (moveVars.rpmAmount*1 == moveVars.GearMinRPM)
		 	{moveVars.currentGear-=1; }

		 	print("g "+moveVars.currentGear);

		}


		if(right)
		{
			blob.AddTorque(25);
		}

		if(left)
		{
			blob.AddTorque(-25);
		}
	}
	blob.set_f32("RPM", moveVars.rpmAmount);
	CleanUp(this, blob, moveVars);
}

//cleanup all vars here - reset clean slate for next frame

void CleanUp(CMovement@ this, CBlob@ blob, CarMoveVars@ moveVars)
{
	//reset all the vars here
	moveVars.thrustFactor = 1.0f;
	moveVars.turnFactor = 1.0f;
	moveVars.stoppingFactor = 1.5f;
}

//TODO: fix flags sync and hitting so we dont need this
bool checkForSolidMapBlob(CMap@ map, Vec2f pos)
{
	CBlob@ _tempBlob; CShape@ _tempShape;
	@_tempBlob = map.getBlobAtPosition(pos);
	if (_tempBlob !is null && _tempBlob.isCollidable())
	{
		@_tempShape = _tempBlob.getShape();
		if (_tempShape.isStatic())
		{
			return true;
		}
	}

	return false;
}
