
//war gamemode logic script

#define SERVER_ONLY

#include "WAR_Structs.as";
#include "RulesCore.as";
#include "RespawnSystem.as";
#include "WAR_PopulateSpawnList.as"
#include "WAR_HUDCommon.as";
//#include "TradingCommon.as";
//#include "Descriptions.as";
//simple config function - edit the variables in the config file
void Config(KartCore@ this)
{
	CRules@ rules = getRules();
	string configstr;
	if (rules.exists("rulesconfig"))
	{
		configstr = rules.get_string("rulesconfig");
		printf("Loading rules vars from " + configstr);
	}

	if (configstr.size() == 0)
	{
		configstr = "../Mods/KAGKarting/Rules/CarRace/kartrace_vars.cfg";
		printf("Vars file not found. Using default vars " + configstr);
	}

	ConfigFile cfg = ConfigFile(configstr);

	//spawn after death time
	s32 warmUpTimeSeconds = cfg.read_s32("warmup_time", 0);
	this.warmUpTime = (getTicksASecond() * warmUpTimeSeconds);
	this.spawnTime = cfg.read_u8("spawn_time", 5);
	this.start_difficulty = cfg.read_u8("start_difficulty", 0);
	this.start_class = cfg.read_string("start_class", "Kar");

	this.introduction = cfg.read_string("introduction", "");
	rules.set_string("introduction", this.introduction);
	rules.Sync("introduction", true);

	this.repeat_map_if_lost = cfg.read_bool("repeat_map_if_lost", false);

	this.minimum_players_in_team = cfg.read_s32("minimum_players_in_team", 1);
	this.scramble_teams = false;

	//how long for the game to play out?
	s32 gameDurationSecs = cfg.read_s32("game_time", -1);
	//	if (sv_test)
	//	{
	//		gameDurationSecs = 0;
	//	}

	if (gameDurationSecs <= 0)
	{
		this.gameTicksLeft = 0;
		rules.set_bool("no timer", true);
	}
	else
	{
		this.gameTicksLeft = (getTicksASecond() * gameDurationSecs);
	}

	CBlob@[] allblobs;
	getBlobs(@allblobs);
	u8 CPCount = 0;
	for (uint i = 0; i < allblobs.length; i++)
	{
		CBlob @b = allblobs[i];
		if(b.getName() == "checkpoint" )
		{
			CPCount++;
		}
	}
	rules.set_u8("CPCount", CPCount);
	rules.set_u8("TotalLaps", cfg.read_u8("laps", 0));
	rules.set_u32("game ticks left", this.gameTicksLeft);
	rules.set_u32("game ticks duration", this.gameTicksLeft);
	rules.Sync("game ticks left", true);
	rules.Sync("game ticks duration", true);

	rules.set_bool("repeat if dead", false);
}

// without "shared" we cannot hot-swap this class :(
// with "shared" we needt o use other function that are "shared" too

shared class KartSpawns : RespawnSystem
{
	KartCore@ mycore;

	WarPlayerInfo@[] spawns;

	bool force;
	s32 nextSpawn;
	bool first;

	void SetCore(RulesCore@ _core)
	{
		RespawnSystem::SetCore(_core);
		@mycore = cast < KartCore@ > (core);
		nextSpawn = 10;
		first = true;
	}

	void Update()
	{
		s32 time = getGameTime();
		if (time % 28 == 0)
		{
			for (uint i = 0; i < spawns.length; i++)
			{
				updatePlayerSpawnTime(spawns[i]);
			}

			// calculate team sizes

			getRules().set_u8("team 0 count", getTeamSize(mycore.teams, 0));
			getRules().set_u8("team 1 count", getTeamSize(mycore.teams, 1));
		}

		if (time > nextSpawn) //each second
		{
			for (uint i = 0; i < spawns.length; i++)
			{
				WarPlayerInfo@ info = spawns[i];
				if (info.wave_delay > 0)
				{
					info.wave_delay--;
				}
			}
			int delta = -1;
			//we do erases in here, and unfortunately don't
			//have any other way to detect them than just looping until nothing more comes out.
			while (delta != 0)
			{
				uint len = spawns.length;
				for (uint i = 0; i < spawns.length; i++)
				{
					WarPlayerInfo@ info = spawns[i];
					DoSpawnPlayer(info);   //check if we should spawn them
				}
				delta = spawns.length - len;
			}

			nextSpawn += 30;
		}
	}

	void updatePlayerSpawnTime(WarPlayerInfo@ w_info)
	{
		WarTeamInfo@ team = cast < WarTeamInfo@ > (core.getTeam(w_info.team));
		//sync player time to them directly
		string propname = "time to spawn " + w_info.username;
		s32 time = getSpawnTime(team, w_info);
		mycore.rules.set_s32(propname, time);
		mycore.rules.SyncToPlayer(propname, getPlayerByUsername(w_info.username));
		propname = "needs respawn hud " + w_info.username;
		mycore.rules.set_bool(propname, (time > s32(getGameTime())));
		mycore.rules.SyncToPlayer(propname, getPlayerByUsername(w_info.username));
	}

	void DoSpawnPlayer(PlayerInfo@ p_info)
	{
		WarPlayerInfo@ w_info = cast < WarPlayerInfo@ > (p_info);

		if (canSpawnPlayer(p_info))
		{
			CPlayer@ player = getPlayerByUsername(p_info.username); // is still connected?
			if (player is null)
			{
				return;
			}
			RemovePlayerFromSpawn(player);
			
			
			CBlob@ spawnBlob;
			CBlob@ playerBlob;
			{
				@spawnBlob = getSpawnBlobs(p_info);
				if (spawnBlob !is null)
				{
					if (spawnBlob.hasTag("migrant"))   // kill the migrant
					{
						spawnBlob.server_Die();
					}
					@playerBlob = SpawnPlayerIntoWorld(spawnBlob.getPosition(), p_info);

					if (playerBlob !is null && spawnBlob.hasTag("bed"))  // send "respawn" cmd
					{
						CBitStream params;
						params.write_netid(playerBlob.getNetworkID());
						spawnBlob.SendCommand(spawnBlob.getCommandID("respawn"), params);
					}
				}
				else if (!mycore.rules.isMatchRunning())
				{
					// create new builder at edge
					if (p_info.team < mycore.teams.length)
					{
						p_info.blob_name = "car"; // force 
						WarTeamInfo@ team = cast < WarTeamInfo@ > (core.getTeam(p_info.team));
						if (team.bedsCount > 0 || !team.under_raid)
						{
							@playerBlob = SpawnPlayerIntoWorld(getSpawnLocation(p_info.team), p_info);
						}
					}
				}
			}

			if (playerBlob !is null)
			{
				//hud
				string propname = "needs respawn hud " + p_info.username;
				mycore.rules.set_bool(propname, false);
				mycore.rules.SyncToPlayer(propname, getPlayerByUsername(p_info.username));

				p_info.spawnsCount++;
			}
			else // search for spawn again
			{
				AddPlayerToSpawn(player);
			}

		}
	}

	bool canSpawnPlayer(PlayerInfo@ p_info)
	{
		WarPlayerInfo@ w_info = cast < WarPlayerInfo@ > (p_info);

		return w_info.wave_delay == 0;
	}

	s32 getSpawnTime(WarTeamInfo@ team, WarPlayerInfo@ w_info)
	{
		return nextSpawn + w_info.wave_delay * 30;
	}

	Vec2f getSpawnLocation(int team)
	{
		CMap@ map = getMap();
		const u16 mapWidth = map.tilemapwidth * map.tilesize;
		const u16 mapHeight = map.tilemapheight * map.tilesize;

		// get slots
		Vec2f respawnPos;
		CBlob@[] blobs;
		map.getBlobsInBox(Vec2f(0,0), Vec2f(mapWidth,mapHeight), @blobs);
		for (uint i = 0; i < blobs.length; i++)
		{
			CBlob@ blob = blobs[i];
			if (blob.getName() == "slot" && blob.get_bool("selected"))
			{
				return blob.getPosition();
			}
		}
		

		f32 side = map.tilesize * 5.0f;
		f32 x = team == 0 ? side : (map.tilesize * map.tilemapwidth - side);
		f32 y = map.tilesize * map.tilemapheight;
		for (uint i = 0; i < map.tilemapheight; i++)
		{
			y -= map.tilesize;
			if (!map.isTileSolid(map.getTile(Vec2f(x, y)))
			        && !map.isTileSolid(map.getTile(Vec2f(x - map.tilesize, y)))
			        && !map.isTileSolid(map.getTile(Vec2f(x + map.tilesize, y)))
			        && !map.isTileSolid(map.getTile(Vec2f(x, y - map.tilesize)))
			        && !map.isTileSolid(map.getTile(Vec2f(x, y - 2 * map.tilesize)))
			        && !map.isTileSolid(map.getTile(Vec2f(x, y - 3 * map.tilesize)))
			   )
				break;
		}
		y -= 32.0f;
		return Vec2f(x, y);
	}

	void RemovePlayerFromSpawn(CPlayer@ player)
	{
		WarPlayerInfo@ info = cast < WarPlayerInfo@ > (core.getInfoFromPlayer(player));
		if (info is null) { warn("KART LOGIC: Couldn't get player info ( in void RemovePlayerFromSpawn(CPlayer@ player) )"); return; }

		int pos = spawns.find(info);
		if (pos != -1)
		{
			spawns.erase(pos);
		}
	}

	void AddPlayerToSpawn(CPlayer@ player)
	{
		RemovePlayerFromSpawn(player);
		if (player.getTeamNum() == core.rules.getSpectatorTeamNum())
			return;

		WarPlayerInfo@ info = cast < WarPlayerInfo@ > (core.getInfoFromPlayer(player));

		if (info is null) { warn("Kart LOGIC: Couldn't get player info  ( in void AddPlayerToSpawn(CPlayer@ player) )"); return; }

		//wave delay used as seconds
		info.wave_delay = 1 + mycore.spawnTime;
		info.spawnpoint = player.getSpawnPoint();

		spawns.push_back(info);
	}

	bool isSpawning(CPlayer@ player)
	{
		WarPlayerInfo@ info = cast < WarPlayerInfo@ > (core.getInfoFromPlayer(player));
		int pos = spawns.find(info);
		return (pos != -1);
	}

	CBlob@ getSpawnBlobs(PlayerInfo@ p_info, bool takeUnderRaid = false)
	{
		CBlob@[] available;
		WarPlayerInfo@ w_info = cast < WarPlayerInfo@ > (p_info);
		if (w_info.deathDistanceToBase <= 0.0f) // first time
		{
			return null;
		}

		Vec2f deathPosition = w_info.deathPosition;

		// get "respawn" bases
		// only if close to them

		PopulateSpawnList(@available, w_info.team, takeUnderRaid);

		if (available.length > 0)
		{
			// pick closest to death position
			while (available.size() > 0)
			{
				f32 closestDist = 999999.9f;
				uint closestIndex = 999;
				for (uint i = 0; i < available.length; i++)
				{
					CBlob @b = available[i];
					Vec2f bpos = b.getPosition();
					const f32 dist = (bpos - w_info.deathPosition).getLength();
					if (dist < closestDist)
					{
						closestDist = dist;
						closestIndex = i;
					}
				}
				if (closestIndex >= 999)
				{
					break;
				}
				return available[closestIndex];
			}
		}

		return null;
	}

};

shared class KartCore : RulesCore
{
	s32 warmUpTime;
	s32 gameDuration;
	s32 spawnTime;
	s32 difficulty;
	s32 start_difficulty;
	f32 lastDifficultyIncPos;
	int intervalSpawnerCount;
	f32 playerFarthestDistance;
	s32 gameTicksLeft;
	string introduction;
	bool repeat_map_if_lost;
	s32 minimum_players_in_team;
	bool scramble_teams;

	int showIntroductionTime;
	int showIntroductionCounter;

	string start_class;

	KartSpawns@ war_spawns;

	KartCore() {}

	KartCore(bool skip)
	{
		super(skip);
	}

	void Setup(CRules@ _rules = null, RespawnSystem@ _respawns = null)
	{
		RulesCore::Setup(_rules, _respawns);
		@war_spawns = cast < KartSpawns@ > (_respawns);
		rules.SetCurrentState(WARMUP);
		server_CreateBlob("Entities/Meta/ChallengeMusic.cfg");

		difficulty = start_difficulty;
		lastDifficultyIncPos = 0.0f;
		intervalSpawnerCount = -1;
		playerFarthestDistance = 150.0f;
		showIntroductionTime = 20 * 30;
		showIntroductionCounter = 0;
	}

	int gametime;
	int startTime;

	void Update()
	{
		const u32 time = getGameTime();
		//HUD
		if (time % 31 == 0)
		{
			updateHUD();			
		}
		if (rules.isGameOver()) { return; }

		int tick = 35;

		if (time % tick == 0)
		{
			UpdatePlayerCounts();
		}

		const bool hasPlayers = false;

		if (rules.isIntermission() || rules.isWarmup()  && (!havePlayers()))
		{
			rules.SetGlobalMessage("Select a slot to start.\nOr wait for more people to join...");
		}
		else if (havePlayers() && rules.isWarmup())
		{
			warmUpTime--;
			if (warmUpTime <= 0)
			{
				rules.SetCurrentState(GAME);
				rules.SetGlobalMessage(rules.get_string("introduction"));

			}
			else
			{
				rules.SetGlobalMessage(rules.get_string("introduction") + "\n\nStarting in... " + (warmUpTime / 30) + "s");
			}
		}
		else if (rules.isMatchRunning())
		{
			if (time % 30 == 0)
			{
				UpdatePlayerCounts();

				// update timer

				if (gameTicksLeft > 0)
				{
					gameTicksLeft -= 30;
					if (gameTicksLeft <= 0)
					{
						// end game - time limit
						rules.SetTeamWon(1);
						rules.SetCurrentState(GAME_OVER);
						rules.SetGlobalMessage("Time passed!");
						sv_mapautocycle = true;
						gameTicksLeft = 0;
					}
					rules.set_u32("game ticks left", gameTicksLeft);
					rules.Sync("game ticks left", true);
					if (gameTicksLeft == 0)
					{ return; }

				}

				if (showIntroductionCounter < showIntroductionTime)
				{
					rules.SetGlobalMessage(rules.get_string("introduction"));
					showIntroductionCounter++;
				}
				else
				{
					rules.SetGlobalMessage("");
				}
			}
		}
		RulesCore::Update(); //update respawns
	}

	void updateHUD()
	{

	}

	bool havePlayers()
	{
		for (uint i = 0; i < teams.length; i++)
		{
			WarTeamInfo@ team = cast < WarTeamInfo@ > (teams[0]);
			if (team.alive_count < 1)
			{
				return false;
			}
		}
		return true;
	}

	//team stuff

	void AddTeam(CTeam@ team)
	{
		WarTeamInfo t(teams.length, team.getName());
		teams.push_back(t);
	}

	void AddPlayer(CPlayer@ player, u8 team = 0, string default_config = "")
	{
		WarPlayerInfo p(player.getUsername(), 0, getStartClass("car"));	// always team 0
		players.push_back(p);
		ChangeTeamPlayerCount(p.team, 1);
	}

	string getStartClass(const string lastClassName)
	{
		bool noBuilder = getTeamSize(teams, 0) <= 1;

		CBlob@[] players;
		getBlobsByTag("player", @players);
		int playersCount = 0;
		for (uint i = 0; i < players.length; i++)
		{
			CBlob@ player = players[i];
		}

		// set it

		if (start_class == "last")
		{
			return lastClassName;
		}
		return start_class;
	}

	void onPlayerDie(CPlayer@ victim, CPlayer@ killer, u8 customData)
	{
		if (victim !is null)
		{
			CBlob@ blob = victim.getBlob();
			if (blob !is null)
			{
				f32 deathDistanceToBase = Maths::Abs(war_spawns.getSpawnLocation(blob.getTeamNum()).x - blob.getPosition().x);
				NotifyDeathPosition(victim, blob.getPosition(), deathDistanceToBase);

				int coins = victim.getCoins();
				server_DropCoins(blob.getPosition(), coins * 0.5f);
				victim.server_setCoins(coins * 0.5f); // lose half coins
			}
		}
	}

	void UpdatePlayerCounts()
	{
		for (uint i = 0; i < teams.length; i++)
		{
			WarTeamInfo@ team = cast < WarTeamInfo@ > (teams[i]);
			//"reset" with migrant count
			team.alive_count = team.migrantCount;
			team.under_raid = false;
		}

		for (uint step = 0; step < players.length; ++step)
		{
			CPlayer@ p = getPlayerByUsername(players[step].username);
			if (p is null) continue;
			CBlob@ player = p.getBlob();
			if (player is null) continue;
			//whew, actually got a blob now..
			if (!player.hasTag("dead"))
			{
				uint teamNum = uint(player.getTeamNum());
				if (teamNum >= 0 && teamNum < teams.length)
				{
					teams[teamNum].alive_count++;
				}
			}
		}
	}

	void NotifyDeathPosition(CPlayer@ player, Vec2f deathPosition, const f32 distance)
	{
		WarPlayerInfo@ info = cast < WarPlayerInfo@ > (getInfoFromPlayer(player));
		if (info is null) { return; }
		info.deathDistanceToBase = distance;
		info.deathPosition = deathPosition;
	}

	bool getPlayerBlobs(CBlob@[]@ playerBlobs)
	{
		for (uint step = 0; step < players.length; ++step)
		{
			CPlayer@ p = getPlayerByUsername(players[step].username);
			if (p is null) continue;

			CBlob@ blob = p.getBlob();
			if (blob !is null)
			{
				playerBlobs.push_back(blob);
			}
		}
		return playerBlobs.size() > 0;
	}
};

//pass stuff to the core from each of the hooks

void onInit(CRules@ this)
{
	Reset(this);
}

void onRestart(CRules@ this)
{
	Reset(this);
}

void Reset(CRules@ this)
{
	printf("Restarting rules script: " + getCurrentScriptName());
	KartSpawns spawns();
	KartCore core(true); //delayed setup
	Config(core);
	core.Setup(this, spawns);
	this.set("core", @core);

	this.set_bool("singleplayer", true);
	this.SetGlobalMessage("");
}

void DeleteSpawnersAtPos(const string &in name, Vec2f pos, const f32 radius)
{
	Vec2f[] knightsPos;
	CMap@ map = getMap();
	if (map.getMarkers(name, knightsPos))
	{
		for (uint i = 0; i < knightsPos.length; i++)
		{
			Vec2f spos = knightsPos[i];
			if ((pos - spos).getLength() < radius)
			{
				map.RemoveMarker(name, i);
				DeleteSpawnersAtPos(name, pos, radius);
				return;
			}
		}
	}
}